@@ -1,6 +1,7 @@
-import typing
 from ast import *
 
+from ..util import CompilingNodeTransformer
+
 """
 Make sure that certain variable names may never be overwritten
 """
@@ -20,7 +21,9 @@ class ForbiddenOverwriteError(ValueError):
     pass
 
 
-class RewriteForbiddenOverwrites(NodeTransformer):
+class RewriteForbiddenOverwrites(CompilingNodeTransformer):
+    step = "Checking for forbidden name overwrites"
+
     def visit_Name(self, node: Name) -> Name:
         if node.ctx == Store() and node.id in FORBIDDEN_NAMES:
             raise ForbiddenOverwriteError(
